var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "path", "./utils", "fs-extra"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.EjectScript = void 0;
    const path_1 = require("path");
    const utils_1 = require("./utils");
    const fs_extra_1 = require("fs-extra");
    const currDir = process.cwd();
    class EjectScript {
        constructor(config) {
            this.config = config;
        }
        copyProject(projectName) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    // Copy project from source to target
                    console.log(`Copying ${projectName} project...`);
                    yield (0, fs_extra_1.copy)(`./apps/${projectName}`, `../${projectName}`, {
                        filter: (src) => {
                            return (!src.includes("node_modules") &&
                                !src.includes(".git") &&
                                !src.includes(".next") &&
                                !src.includes(".expo") &&
                                !src.includes(".idea") &&
                                !src.includes(".DS_Store"));
                        },
                    });
                }
                catch (err) {
                    console.error(`Error while copying ${projectName} project`, err);
                }
            });
        }
        copyPackageJSON(projectName, packageJSONPath) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    console.log(`Copying package.json in ${projectName}...`, packageJSONPath);
                    let useProjectPackageJSON = require((0, path_1.resolve)(currDir, `../${projectName}/package.json`));
                    const currentProjectPackageJSON = require(packageJSONPath);
                    if (!currentProjectPackageJSON) {
                        console.log("No package.json found in source directory!");
                        return;
                    }
                    useProjectPackageJSON.dependencies = Object.assign(Object.assign({}, currentProjectPackageJSON.dependencies), useProjectPackageJSON.dependencies);
                    useProjectPackageJSON.devDependencies = Object.assign(Object.assign({}, currentProjectPackageJSON.devDependencies), useProjectPackageJSON.devDependencies);
                    delete useProjectPackageJSON.dependencies["@/components"];
                    delete useProjectPackageJSON.dependencies["@/screens"];
                    yield (0, fs_extra_1.writeFile)((0, path_1.resolve)(currDir, `../${projectName}/package.json`), JSON.stringify(useProjectPackageJSON, null, 2));
                }
                catch (err) {
                    console.error("Error while copying package.json", err);
                }
            });
        }
        copyComponents(projectName) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    // Copy components from source to target
                    console.log("Copying ui components...");
                    yield (0, utils_1.copyDirectoryFromSourceToTarget)({
                        sourceDirectory: this.config.components + "/ui",
                        copyDestination: `../${projectName}/components/ui`,
                    });
                    console.log("Copying custom components...");
                    yield (0, utils_1.copyDirectoryFromSourceToTarget)({
                        sourceDirectory: this.config.components + "/custom",
                        copyDestination: `../${projectName}/components/custom`,
                    });
                    const packageJSONPath = (0, path_1.resolve)(currDir, `${this.config.components}/package.json`);
                    yield this.copyPackageJSON(projectName, packageJSONPath);
                }
                catch (err) {
                    console.error("Error while copying components", err);
                }
            });
        }
        copyScreens(projectName) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    yield (0, utils_1.copyDirectoryFromSourceToTarget)({
                        sourceDirectory: this.config.screens,
                        copyDestination: `../${projectName}/screens`,
                    });
                    const packageJSONPath = (0, path_1.resolve)(currDir, `${this.config.screens}/package.json`);
                    yield this.copyPackageJSON(projectName, packageJSONPath);
                }
                catch (err) {
                    console.error("Error while copying screens", err);
                }
            });
        }
        updateTsConfig(projectName) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    // Update tsconfig.json
                    const tsConfigPath = (0, path_1.resolve)(currDir, `../${projectName}/tsconfig.json`);
                    const tsConfig = require(tsConfigPath);
                    if (!tsConfig.compilerOptions) {
                        tsConfig.compilerOptions = {};
                    }
                    if (!tsConfig.compilerOptions.paths) {
                        tsConfig.compilerOptions.paths = {};
                    }
                    if (!tsConfig.compilerOptions.paths["@/*"]) {
                        tsConfig.compilerOptions.paths["@/*"] = ["./*"];
                    }
                    tsConfig.compilerOptions.paths = Object.assign({}, tsConfig.compilerOptions.paths);
                    yield (0, fs_extra_1.writeFile)(tsConfigPath, JSON.stringify(tsConfig, null, 2));
                }
                catch (err) {
                    console.error("Error while updating tsconfig.json", err);
                }
            });
        }
        updateTailwindConfig(projectName) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    // Update tailwind.config.js
                    const tailwindConfigPath = (0, path_1.resolve)(currDir, `../${projectName}/tailwind.config.js`);
                    let tailwindConfig = require(tailwindConfigPath);
                    if (!tailwindConfig) {
                        console.log("No tailwind.config.js found in source directory!");
                        return;
                    }
                    tailwindConfig.content.push("./components/**/*.{js,jsx,ts,tsx}", "./screens/**/*.{js,jsx,ts,tsx}");
                    tailwindConfig.content = tailwindConfig.content.filter((item) => item !== "../../packages/components/**/*.{js,jsx,ts,tsx}" &&
                        item !== "../../packages/screens/**/*.{js,jsx,ts,tsx}");
                    yield (0, fs_extra_1.writeFile)(tailwindConfigPath, `module.exports = ${JSON.stringify(tailwindConfig, null, 2)}`);
                }
                catch (err) {
                    console.error("Error while updating tailwind.config.js", err);
                }
            });
        }
        eject(projectName) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    console.log("Running eject script...");
                    yield this.copyComponents(projectName);
                    console.log("Components copied successfully!");
                    console.log("Copying screens...");
                    yield this.copyScreens(projectName);
                    console.log("Screens copied successfully!");
                    console.log("Updating tsconfig.json...");
                    yield this.updateTsConfig(projectName);
                    console.log("tsconfig.json updated okay!");
                    console.log("Updating tailwind.config.js...");
                    yield this.updateTailwindConfig(projectName);
                    console.log("tailwind.config.js updated successfully!");
                }
                catch (err) {
                    console.error("Error while running eject script", err);
                }
            });
        }
        run() {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const projectName = this.config.project;
                    if (projectName && projectName !== "all") {
                        console.log("Copying project...");
                        yield this.copyProject(projectName);
                        yield this.eject(projectName);
                    }
                    else {
                        console.log("Copying next project...");
                        yield this.copyProject("next");
                        yield this.eject("next");
                        console.log("Copying expo project...");
                        yield this.copyProject("expo-app");
                        yield this.eject("expo-app");
                    }
                }
                catch (err) {
                    console.error("Error while running eject script", err);
                }
            });
        }
    }
    exports.EjectScript = EjectScript;
});
